# Hands-on 1: Seasonal forecast verification

## Goal
Learn how to use the startR workflow to finish a piece of analysis, including defining and pre-processing the desired data,
defining the operation, building the workflow, and executing the operation.

In this use case, the continuous ranked probability score (CRPS) and the root mean square error skill score (RMSSS) are computed to verify the forecast.
To make the process faster, the required data size is small here so we can run the execution on workstation.

## 1. Define data
The first step of the analysis is use Start() to identify the data needed.

Hints:  
(1) The data paths use two '$' as the wildcard, and it can be given any names you like.  
(2) The variable we want to use is 'tas'.   
(3) The required time (sdate) period is November 1991 to 2011.  
(4) Read global data (i.e., full spatial grids.)  
(5) Take only the first time step.  
(6) To make the data smaller, we only take the first two ensemble members from the experimental data.  
(7) Because we are going to use the whole startR workflow, we only need to create a pointer to the data repository rather than retrieve it to the workstation.  

```r
library(startR)

# exp data
  # Use this one if on workstation or nord3 (have access to /esarchive)
  repos_exp <- paste0('/esarchive/exp/ecmwf/system5c3s/monthly_mean/',
                      '$var$_f6h/$var$_$sdate$.nc')
  # Use this one if on Marenostrum4 and log in with PATC2021 account
  repos_exp <- paste0('/gpfs/scratch/nct00/nct00016/d3_02_R-hands_on/tmp-esarchive/',
                      'exp/ecmwf/system4_m1/monthly_mean/',
                      '$var$_f6h/$var$_$sdate$.nc')

  sdates_exp <- sapply(1991:2011, function(x) paste0(x, '1101'))

  exp <- Start(dat = repos_exp,
               var = 'tas',
               sdate = sdates_exp,
               time = indices(1),
               ensemble = 'all',
               latitude = 'all',
               longitude = 'all',
               synonims = list(longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')),
               return_vars = list(latitude = NULL, longitude = NULL, time = 'sdate'),
               retrieve = FALSE)

# obs data
  # Use this one if on workstation or nord3 (have access to /esarchive)
  repos_obs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r360x180/',
                      '$var$_$sdate$.nc')
  # Use this one if on Marenostrum4 and log in with PATC2022 account
  repos_obs <- paste0('/gpfs/scratch/nct00/nct00016/d3_02_R-hands_on/tmp-esarchive/',
                      'recon/ecmwf/erainterim/monthly_mean/',
                      '$var$_f6h/$var$_$sdate$.nc')

  sdates_obs <- sapply(1991:2011, function(x) paste0(x, '11'))

  obs <- Start(dat = repos_obs,
               var = 'tas',
               sdate = sdates_obs,
               time = indices(1),
               latitude = 'all',
               longitude = 'all',
               synonims = list(longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')),
               return_vars = list(latitude = NULL, longitude = NULL, time = 'sdate'),
               retrieve = FALSE)
```

Question:  

1. What are the dimensions of these two data?
```r
attr(exp, 'Dimensions')
      dat       var     sdate      time  ensemble  latitude longitude 
        1         1        21         1        25       181       360 

attr(obs, 'Dimensions')
      dat       var     sdate      time  latitude longitude 
        1         1        21         1       181       360 
```

2. What is the size of these two data?
exp: 261Mb; obs: 10.4Mb as shown on screen from Start() call.


## 2. Define operation and workflow
It is recommended to define the function and write Step() together, because the latter one helps you clarify the input and output dimensions of the function.

In the self-defined function, we want to use the two functions, 's2dv::CRPSS' and 's2dv::RMSSS', to calculate the skill scores.
You can check the functions by typing `?s2dv::CRPSS` and `?s2dv::RMSSS`, or on [s2dv GitLab](https://earth.bsc.es/gitlab/es/s2dv/-/blob/master/R/).  

Hint:  
(1) The self-defined function is for exp and obs data. Therefore, target_dims and output_dims in Step() should be a list with two vector elements.
(2) The essential dimensions are "ensemble" and "sdate" for `exp`, and "sdate" for `obs`. These dimensions should be the 'target_dims' in Step().  
(3) To return the two skill scores ('crpss' and 'rmsss') together, put them in a list with two elements.  
(4) What are the dimensions of the outputs? These dimensions should be 'output_dims' in Step(). 
(5) The first input of AddStep() should also be a list containing exp and obs.

```r
  # self-defined function
  skill_func <- function(exp, obs) {
    # exp: [ensemble, sdate]
    # obs: [sdate]

    # Calculate CRPSS
    crpss <- s2dv::CRPSS(exp, obs, memb_dim = 'ensemble')

    # Calculate RMSSS
    ## ensemble mean
    exp_ens <- s2dv::MeanDims(exp, 'ensemble', na.rm = T)
    rmsss <- s2dv::RMSSS(exp_ens, obs, time_dim = 'sdate', dat_dim = NULL)

    return(list(crpss = crpss$crpss, crpss_sign = crpss$sign, rmsss = rmsss$rmsss, rmsss_pval = rmsss$p.val))
  }
  step <- Step(skill_func, target_dims = list(exp = c('ensemble', 'sdate'), obs = c('sdate')),
               output_dims = list(crpss = NULL, crpss_sign = NULL, rmsss = NULL, rmsss_pval = NULL))
  wf <- AddStep(list(exp, obs), step)
```

Question:  
1. Which dimensions are used in operation? Which dimensions are free?
'ensemble' and 'sdate' are used in operation. The other dimensions (dat, var, time, latitude, and longitude) 
are not necessary for the functions. These free dimensions will be subsetted by multiApply during the execution.

2. Can we use more dimensions as target_dims? What are the disadvantages?  
It still works if putting more dimensions as target_dims, but we will lose the choices for chunking in the next step. Also, the lighter the function is, the quicker the operation runs.  



## 3. Execute locally
To avoid potential technical problems in the connection and configuration,
we choose to run the execution locally.
Noted that it is recommended to submit jobs to HPCs if the data size is big or
the operation is heavy.

Hint:  
(1) Use the free dimensions (i.e., those are not 'target_dims' in Step()) to chunk.  
(2) It is safe to divide the data into pieces of which the size each is 1/2-1/3 of RAM. In this case, 
the data size is only around 270Mb, so you can play with it without worrying about crashing the terminal.  

```r
  res <- Compute(wf$crpss,
                 chunks = list(latitude = 2, longitude = 2))
```

## 4. Check the results

1. Check the list structure using function `str()`. What are the items in the list?
```r
str(res)
List of 4
 $ crpss     : num [1, 1, 1, 1:181, 1:360] -13.6 -12.2 -10.6 -10.2 -10.7 ...
 $ crpss_sign: logi [1, 1, 1, 1:181, 1:360] TRUE TRUE TRUE TRUE TRUE TRUE ...
 $ rmsss     : num [1, 1, 1, 1:181, 1:360] 0.943 0.943 0.944 0.943 0.941 ...
 $ rmsss_pval: num [1, 1, 1, 1:181, 1:360] 0 0 0 0 0 0 0 0 0 0 ...
  ..
```

2. What are the dimensions of each item?
```r
dim(res$crpss)
      dat       var      time  latitude longitude 
        1         1         1       181       360 
dim(res$crpss_sign)
      dat       var      time  latitude longitude
        1         1         1       181       360
dim(res$rmsss)
      dat       var      time  latitude longitude
        1         1         1       181       360
dim(res$rmsss_pval)
      dat       var      time  latitude longitude 
        1         1         1       181       360 
```

3. Plot the maps  

Use `s2dv::PlotEquiMap` or other visualization tools to plot the map.
```r
  # Get latitude and longitude from the attributes
  lat <- as.vector(attr(exp, 'Variables')$common$latitude)
  lon <- as.vector(attr(exp, 'Variables')$common$longitude)

  # Set the color bar 
  brks_crps <- seq(0, max(res$crps, na.rm = TRUE), length.out = 21)
  brks_rmsss <- seq(min(res$rmsss, na.rm = TRUE), max(res$rmsss, na.rm = TRUE), length.out = 21)

  library(s2dv)
  # Plot crps
  PlotEquiMap(res$crps[1, 1, 1, , ], lon, lat,
              cols = heat.colors(length(brks_crps) - 1)[(length(brks_crps) - 1):1], 
              brks = brks_crps,
              filled.continents = FALSE, triangle_ends = c(TRUE, TRUE),
              toptitle = 'ECMWF monthly mean tas CRPS 2012-2016', title_scale = 0.6,
              fileout = '~/handson_1_fig_crps.png')

  # Plot rmsss
  PlotEquiMap(res$rmsss[1, 1, 1, , ], lon, lat,
              color_fun = clim.palette('yellowred'), brks = brks_rmsss,
              filled.continents = FALSE, triangle_ends = c(TRUE, TRUE),
              toptitle = 'ECMWF monthly mean tas RMSSS 2012-2016', title_scale = 0.6,
              fileout = '~/handson_1_fig_rmsss.png')
```

![CRPS map](./Figures/handson_1_fig_crps.png)
 
![RMSSS map](./Figures/handson_1_fig_rmsss.png)

